@@ -184,6 +184,39 @@ ncm_dataset_new (void)
   return dset;
 }
 
+/**
+ * ncm_dataset_new_list:
+ * @data0: first #NcmData to be added.
+ * @...: a NULL ended list of #NcmData
+ *
+ * Creates a new #NcmDataset object and adds a NULL ended list
+ * of #NcmData. 
+ *
+ * Returns: a new #NcmDataset.
+ */
+NcmDataset *
+ncm_dataset_new_list (gpointer data0, ...)
+{
+  va_list ap;
+  NcmDataset *dset = ncm_dataset_new ();
+
+  if (data0 != NULL)
+  {
+    NcmData *data = NULL;
+    
+    va_start (ap, data0);
+
+    ncm_dataset_append_data (dset, data0);
+        
+    while ((data = va_arg (ap, NcmData *)) != NULL)
+      ncm_dataset_append_data (dset, data);
+
+    va_end (ap);
+  }
+  
+  return dset;
+}
+
 /**
  * ncm_dataset_ref:
  * @dset: pointer to type defined by #NcmDataset
@@ -269,6 +302,7 @@ ncm_dataset_append_data (NcmDataset *dset, NcmData *data)
 {
   gboolean enable = (dset->bstype != NCM_DATASET_BSTRAP_DISABLE) ? TRUE : FALSE;
 
+  g_assert (NCM_IS_DATA (data));
   ncm_obj_array_add (dset->oa, G_OBJECT (data));
 
   if (enable)
@@ -390,7 +424,7 @@ ncm_dataset_get_data (NcmDataset *dset, guint n)
 NcmData *
 ncm_dataset_peek_data (NcmDataset *dset, guint n)
 {
-  g_assert (n < dset->oa->len);
+  g_assert_cmpuint (n, <, dset->oa->len);
   return NCM_DATA (ncm_obj_array_peek (dset->oa, n));
 }
 
@@ -960,7 +994,7 @@ ncm_dataset_m2lnL_vec (NcmDataset *dset, NcmMSet *mset, NcmVector *m2lnL_v)
 {
   guint i;
 
-  g_assert (ncm_vector_len (m2lnL_v) >= dset->oa->len);
+  g_assert_cmpuint (ncm_vector_len (m2lnL_v), >=, dset->oa->len);
 
   for (i = 0; i < dset->oa->len; i++)
   {
@@ -980,38 +1014,6 @@ ncm_dataset_m2lnL_vec (NcmDataset *dset, NcmMSet *mset, NcmVector *m2lnL_v)
   return;
 }
 
-/**
- * ncm_dataset_m2lnL_i_val:
- * @dset: a #NcmLikelihood
- * @mset: a #NcmMSet
- * @i: an integer
- * @m2lnL_i: (out): FIXME
- *
- * Get the value of the @i-th data in the dataset.
- *
- */
-void
-ncm_dataset_m2lnL_i_val (NcmDataset *dset, NcmMSet *mset, guint i, gdouble *m2lnL_i)
-{
-  *m2lnL_i = 0.0;
-
-  g_assert_cmpuint (i, <, dset->oa->len);
-  {
-    NcmData *data = ncm_dataset_peek_data (dset, i);
-
-    if (!NCM_DATA_GET_CLASS (data)->m2lnL_val)
-      g_error ("ncm_dataset_m2lnL_val: %s dont implement m2lnL", G_OBJECT_TYPE_NAME (data));
-    else
-    {
-      ncm_data_prepare (data, mset);
-      NCM_DATA_GET_CLASS (data)->m2lnL_val (data, mset, m2lnL_i);
-    }
-  }
-
-  return;
-}
-
-
 /**
  * ncm_dataset_m2lnL_grad:
  * @dset: a #NcmLikelihood.
@@ -1023,9 +1025,9 @@ ncm_dataset_m2lnL_i_val (NcmDataset *dset, NcmMSet *mset, guint i, gdouble *m2ln
 void
 ncm_dataset_m2lnL_grad (NcmDataset *dset, NcmMSet *mset, NcmVector *grad)
 {
+  const guint fparams_len = ncm_mset_fparams_len (mset);
+  NcmVector *grad_i       = ncm_vector_new (fparams_len);
   guint i;
-  guint free_params_len = ncm_mset_fparams_len (mset);
-  NcmVector *grad_i = ncm_vector_new (free_params_len);
 
   ncm_vector_set_zero (grad);
 
@@ -1060,9 +1062,9 @@ ncm_dataset_m2lnL_grad (NcmDataset *dset, NcmMSet *mset, NcmVector *grad)
 void
 ncm_dataset_m2lnL_val_grad (NcmDataset *dset, NcmMSet *mset, gdouble *m2lnL, NcmVector *grad)
 {
+  const guint fparams_len = ncm_mset_fparams_len (mset);
+  NcmVector *grad_i       = ncm_vector_new (fparams_len);
   guint i;
-  guint free_params_len = ncm_mset_fparams_len (mset);
-  NcmVector *grad_i = ncm_vector_new (free_params_len);
 
   ncm_vector_set_zero (grad);
   *m2lnL = 0.0;
@@ -1090,3 +1092,64 @@ ncm_dataset_m2lnL_val_grad (NcmDataset *dset, NcmMSet *mset, gdouble *m2lnL, Ncm
 
   ncm_vector_free (grad_i);
 }
+
+/**
+ * ncm_dataset_m2lnL_i_val:
+ * @dset: a #NcmLikelihood
+ * @mset: a #NcmMSet
+ * @i: an integer
+ * @m2lnL_i: (out): FIXME
+ *
+ * Get the value of the @i-th data in the dataset.
+ *
+ */
+void
+ncm_dataset_m2lnL_i_val (NcmDataset *dset, NcmMSet *mset, guint i, gdouble *m2lnL_i)
+{
+  *m2lnL_i = 0.0;
+
+  g_assert_cmpuint (i, <, dset->oa->len);
+  {
+    NcmData *data = ncm_dataset_peek_data (dset, i);
+
+    if (!NCM_DATA_GET_CLASS (data)->m2lnL_val)
+      g_error ("ncm_dataset_m2lnL_val: %s dont implement m2lnL", G_OBJECT_TYPE_NAME (data));
+    else
+    {
+      ncm_data_prepare (data, mset);
+      NCM_DATA_GET_CLASS (data)->m2lnL_val (data, mset, m2lnL_i);
+    }
+  }
+
+  return;
+}
+
+/**
+ * ncm_dataset_fisher_matrix:
+ * @dset: a #NcmDataset
+ * @mset: a #NcmMSet
+ * @IM: (out): The fisher matrix
+ *
+ * Calculates the Fisher-information matrix @I adding
+ * the individual ones from each #NcmData in @dset.
+ * 
+ */
+void 
+ncm_dataset_fisher_matrix (NcmDataset *dset, NcmMSet *mset, NcmMatrix **IM)
+{
+  const guint fparams_len = ncm_mset_fparams_len (mset);
+  NcmMatrix *IM0          = ncm_matrix_new (fparams_len, fparams_len);
+  guint i;
+
+  *IM = ncm_matrix_new (fparams_len, fparams_len);
+
+  ncm_matrix_set_zero (*IM);
+
+  for (i = 0; i < dset->oa->len; i++)
+  {
+    NcmData *data = ncm_dataset_peek_data (dset, i);
+    ncm_data_fisher_matrix (data, mset, &IM0);
+
+    ncm_matrix_add (*IM, IM0);
+  }
+}
